@@ -34,14 +34,15 @@
 #include "yelp-base.h"
 
 typedef struct {
-	YelpBase     *base;
+	YelpBase    *base;
 	GtkTreeIter *parent;
 } ForeachData;
 
 struct _YelpBasePriv {
-	GNode         *toc_tree;
+	GNode  *toc_tree;
 
-	GSList        *windows;
+	GList  *index;
+	GSList *windows;
 };
 
 
@@ -75,6 +76,36 @@ impl_Yelp_newWindow (PortableServer_Servant  servant,
 	}
 }
 
+static GNOME_Yelp_WindowList *
+impl_Yelp_getWindows (PortableServer_Servant  servant,
+		      CORBA_Environment      *ev)
+{
+	YelpBase              *base;
+	YelpBasePriv          *priv;
+	GNOME_Yelp_WindowList *list;
+	gint                   len, i;
+	GSList                *node;
+	const gchar           *url;
+	
+	base = YELP_BASE (bonobo_object (servant));
+	priv = base->priv;
+
+	len  = g_slist_length (priv->windows);
+	
+	list = GNOME_Yelp_WindowList__alloc ();
+	list->_buffer = CORBA_sequence_CORBA_string_allocbuf (len);
+	list->_length = len;
+	list->_maximum = len;
+	CORBA_sequence_set_release (list, CORBA_TRUE);
+	
+	for (node = priv->windows, i = 0; node; node = node->next, i++) {
+		url = yelp_window_get_current_uri (YELP_WINDOW (node->data));
+		list->_buffer[i] = CORBA_string_dup (url);
+	}
+	
+	return list;
+}
+
 static void
 yelp_base_init (YelpBase *base)
 {
@@ -83,9 +114,9 @@ yelp_base_init (YelpBase *base)
         priv = g_new0 (YelpBasePriv, 1);
         
 	priv->toc_tree = g_node_new (NULL);
-
-	priv->windows = NULL;
-        base->priv    = priv;
+	priv->index    = NULL;
+	priv->windows  = NULL;
+        base->priv     = priv;
 }
 
 static void
@@ -95,7 +126,8 @@ yelp_base_class_init (YelpBaseClass *klass)
 	
 	parent_class = gtk_type_class (PARENT_TYPE);
 
-	epv->newWindow = impl_Yelp_newWindow;
+	epv->newWindow  = impl_Yelp_newWindow;
+	epv->getWindows = impl_Yelp_getWindows;
 }
 
 static void
@@ -130,14 +162,15 @@ yelp_base_window_finalized_cb (YelpBase *base, YelpWindow *window)
 YelpBase *
 yelp_base_new (void)
 {
-        YelpBase *base;
-	gboolean  result;
-
+        YelpBase     *base;
+	YelpBasePriv *priv;
+	
         base = g_object_new (YELP_TYPE_BASE, NULL);
-
-	result = yelp_scrollkeeper_init (base->priv->toc_tree);
-	result = yelp_man_init (base->priv->toc_tree);
-	result = yelp_info_init (base->priv->toc_tree);
+	priv = base->priv;
+	
+	yelp_scrollkeeper_init (priv->toc_tree, &priv->index);
+	yelp_man_init (base->priv->toc_tree, &priv->index);
+	yelp_info_init (base->priv->toc_tree, &priv->index);
 	
         return base;
 }
@@ -152,7 +185,7 @@ yelp_base_new_window (YelpBase *base)
 
 	priv = base->priv;
         
-        window = yelp_window_new (base->priv->toc_tree);
+        window = yelp_window_new (priv->toc_tree, priv->index);
         
 	priv->windows = g_slist_prepend (priv->windows, window);
 
@@ -160,7 +193,6 @@ yelp_base_new_window (YelpBase *base)
 			   (GWeakNotify) yelp_base_window_finalized_cb,
 			   base);
 	
-
 	g_signal_connect (window, "new_window_requested",
 			  G_CALLBACK (yelp_base_new_window_cb),
 			  base);
