@@ -40,30 +40,10 @@
 
 #define d(x)
 
-typedef struct _StreamData StreamData;
-
-static void yh_init                  (YelpHtml           *html);
-static void yh_class_init            (YelpHtmlClass      *klass);
-static void yh_link_clicked_cb       (HtmlDocument          *doc, 
-				      const gchar           *url, 
-				      YelpHtml              *html);
-static void yh_stream_cancel         (HtmlStream            *stream, 
-				      gpointer               user_data, 
-				      gpointer               cancel_data);
-static void yh_url_requested_cb      (HtmlDocument          *doc,
-				      const gchar           *uri,
-				      HtmlStream            *stream,
-				      gpointer               data);
-
-#define BUFFER_SIZE 16384
-
-enum {
-	URL_SELECTED,
-	LAST_SIGNAL
-};
-
-static gint        signals[LAST_SIGNAL] = { 0 };
-static GHashTable *cache_table = NULL;
+typedef enum {
+	MAN,
+	INFO
+} DocType;
 
 struct _YelpHtmlPriv {
 	HtmlView     *view;
@@ -75,13 +55,58 @@ struct _YelpHtmlPriv {
 };
 
 typedef struct {
-	YelpHtml       *html;
-	gchar          *buffer;
-	GnomeVFSHandle *handle;
-	HtmlStream     *stream;
-	gboolean        is_doc;
+	YelpHtml   *html;
+	HtmlStream *stream;
+	gchar      *section;
 } ReadData;
 
+static void      yh_init                  (YelpHtml           *html);
+static void      yh_class_init            (YelpHtmlClass      *klass);
+static int       yelp_html_do_write       (void               *context,
+					   const char         *buffer,
+					   int                 len);
+static int       yelp_html_do_close       (void               *context);
+
+static void      yelp_html_do_docbook     (YelpHtml           *html,
+					   const gchar        *docpath,
+					   const gchar        *section,
+					   GError            **error);
+static gboolean  yelp_html_io_watch_cb    (GIOChannel         *iochannel, 
+					   GIOCondition        cond, 
+					   ReadData           *read_data);
+
+static void      yelp_html_do_maninfo     (YelpHtml           *html,
+					   HtmlStream         *stream,
+					   DocType             type, 
+					   const gchar        *uri,
+					   GError            **error);
+static void      yelp_html_do_html        (YelpHtml           *html,
+					   HtmlStream         *stream,
+					   const gchar        *uri,
+					   const gchar        *section,
+					   GError            **error);
+static void      yh_url_requested_cb      (HtmlDocument       *doc,
+					   const gchar        *uri,
+					   HtmlStream         *stream,
+					   gpointer            data);
+static void      yh_stream_cancel         (HtmlStream         *stream, 
+					   gpointer            user_data, 
+					   gpointer            cancel_data);
+
+static void      yh_link_clicked_cb       (HtmlDocument       *doc, 
+					   const gchar        *url, 
+					   YelpHtml           *html);
+
+#define BUFFER_SIZE 16384
+
+enum {
+	URL_SELECTED,
+	LAST_SIGNAL
+};
+
+static gint        signals[LAST_SIGNAL] = { 0 };
+static GHashTable *cache_table = NULL;
+
 GType
 yelp_html_get_type (void)
 {
@@ -212,15 +237,19 @@ yelp_html_do_close (void *context)
 }
 
 static void
-yelp_html_do_docbook (YelpHtml *html, const gchar *uri, GError **error)
+yelp_html_do_docbook (YelpHtml *html, const gchar *docpath, 
+		      const gchar *section, GError **error)
 {
 	xmlOutputBufferPtr  buf;
 	YelpHtmlPriv       *priv;
-
+	gchar              *uri;
+	
 	g_return_if_fail (YELP_IS_HTML (html));
 	
 	priv = html->priv;
 
+	uri = g_strconcat (docpath, "?", section, NULL);
+
 	buf = xmlAllocOutputBuffer (NULL);
 		
 	buf->writecallback = yelp_html_do_write;
@@ -228,25 +257,69 @@ yelp_html_do_docbook (YelpHtml *html, const gchar *uri, GError **error)
 	buf->context       = html;
 		
 	yelp_db2html_convert (uri, buf, error);
+
+	g_free (uri);
 }
 
 static gboolean
-yelp_html_idle_read (gpointer data)
+yelp_html_io_watch_cb (GIOChannel   *iochannel, 
+		       GIOCondition  cond, 
+		       ReadData     *read_data)
 {
-	ReadData         *read_data = (ReadData *)data;
-	GnomeVFSFileSize  read_len;
-	GnomeVFSResult    result;
-	
-	result = gnome_vfs_read (read_data->handle, 
-				 read_data->buffer,
-				 BUFFER_SIZE, 
-				 &read_len);
+	YelpHtmlPriv *priv;
+	static gchar  buffer[BUFFER_SIZE];
+	guint         n;
+	gboolean      finished = FALSE;
+	GIOStatus     status;
 
-	if (result == GNOME_VFS_OK) {
-		yelp_html_do_write (read_data->html, 
-				    read_data->buffer, 
-				    read_len);
-	} else {
+	g_return_val_if_fail (read_data != NULL, FALSE);
+	g_return_val_if_fail (YELP_IS_HTML (read_data->html), FALSE);
+
+	priv = read_data->html->priv;
+
+	if (cond & G_IO_IN) {
+ 		d(g_print ("Read available\n"));
+		
+		status = g_io_channel_read_chars (iochannel,
+						  buffer,
+						  BUFFER_SIZE,
+						  &n,
+						  NULL);
+
+		if (status == G_IO_STATUS_NORMAL) {
+			yelp_html_do_write (read_data->html, buffer, n);
+		} 
+		else if (status == G_IO_STATUS_EOF ||
+			 status == G_IO_STATUS_ERROR) {
+			if (n > 0) {
+				yelp_html_do_write (read_data->html, 
+						    buffer, n);
+			}
+
+			finished = TRUE;
+		}
+	}
+
+	if (cond & G_IO_HUP || finished) {
+		html_stream_close (read_data->stream);
+		
+		d(g_print ("Close\n"));
+
+		g_io_channel_unref (iochannel);
+		
+		gtk_adjustment_set_value (
+			gtk_layout_get_vadjustment (GTK_LAYOUT (priv->view)),
+			0);
+
+		gdk_window_set_cursor (GTK_WIDGET (priv->view)->window, NULL);
+
+		if (read_data->section) {
+			html_view_jump_to_anchor (priv->view, 
+						  read_data->section);
+			g_free (read_data->section);
+		}
+
+		g_free (read_data);
 		return FALSE;
 	}
 
@@ -254,76 +327,88 @@ yelp_html_idle_read (gpointer data)
 }
 
 static void
-yelp_html_idle_read_end (gpointer data)
+yelp_html_do_maninfo (YelpHtml *html, HtmlStream *stream,
+		      DocType type, const gchar *uri, GError **error)
 {
-	ReadData     *read_data = (ReadData *)data;
-	YelpHtml     *html;
-	YelpHtmlPriv *priv;
+	gchar       *command_line = NULL;
+	gchar      **argv = 0;
+	gint         output_fd;
+	GIOChannel  *io_channel;
+	ReadData    *read_data;
 	
-	html = read_data->html;
-	priv = html->priv;
+	d(g_print ("entering from maninfo: %d\n", output_fd));
+
+	switch (type) {
+	case MAN:
+		command_line = g_strdup_printf ("gnome2-man2html %s", uri);
+		break;
+	case INFO:
+		command_line = g_strdup_printf ("gnome2-info2html %s", uri);
+		break;
+	default:
+		g_warning ("Non-supported doctype");
+		return;
+	};
 	
-	gnome_vfs_close (read_data->handle);
-	
-	html_stream_close (read_data->stream);
-
-	d(g_print ("Close\n"));
-
-	if (read_data->is_doc) {
-		gtk_adjustment_set_value (
-			gtk_layout_get_vadjustment (GTK_LAYOUT (priv->view)),
-			0);
-	}
-
-	gdk_window_set_cursor (GTK_WIDGET (priv->view)->window, NULL);
-
-	g_free (read_data);
-}
-
-static void
-yelp_html_do_non_docbook (YelpHtml *html, HtmlStream *stream, 
-			  const gchar *uri, gboolean is_doc, 
-			  GError **error)
-{
-	GnomeVFSHandle *handle;
-	GnomeVFSResult  result;
-	gchar           buffer[BUFFER_SIZE];
-	GTimer         *timer;
-	ReadData       *read_data;
-	
-	g_return_if_fail (YELP_IS_HTML (html));
-	
-	d(g_print ("Non docbook: %s\n", uri));
-
-	timer = g_timer_new ();
-
-	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
-
-	d(g_print ("Opening took: %f\n", g_timer_elapsed (timer, 0)));
-	
-	if (result != GNOME_VFS_OK) {
-		g_set_error (error,
-			     YELP_ERROR,
-			     YELP_ERROR_FAILED_OPEN,
-			     _("Failed to open document '%s'"),
-			     uri);
+	if (!g_shell_parse_argv (command_line, NULL, &argv, error)) {
 		return;
 	}
 
-	g_timer_start (timer);
+	g_free (command_line);
 
-	read_data = g_new0 (ReadData, 1);
+	d(g_print ("spawning in maninfo\n"));
 	
-	read_data->html      = html;
-	read_data->buffer    = buffer;
-	read_data->handle    = handle;
-	read_data->stream    = stream;
-	read_data->is_doc    = is_doc;
+	if (!g_spawn_async_with_pipes (NULL, argv, NULL, 
+				       G_SPAWN_STDERR_TO_DEV_NULL | G_SPAWN_SEARCH_PATH,
+				       NULL, NULL, NULL, NULL, 
+				       &output_fd, NULL, error)) {
+		return;
+	}
+	
+	d(g_print ("spawned in maninfo: %d\n", output_fd));
 
-	g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, 
-			 yelp_html_idle_read,
-			 read_data,
-			 yelp_html_idle_read_end);
+	g_strfreev (argv);
+
+	io_channel = g_io_channel_unix_new (output_fd);
+	
+	read_data = g_new (ReadData, 1);
+	read_data->html    = html;
+	read_data->stream  = stream;
+	read_data->section = NULL;
+	
+	g_io_add_watch (io_channel, G_IO_IN | G_IO_HUP, 
+			(GIOFunc) yelp_html_io_watch_cb,
+			read_data);
+
+	d(g_print ("returning from maninfo: %d\n", output_fd));
+}
+
+static void
+yelp_html_do_html (YelpHtml     *html,
+		   HtmlStream   *stream,
+		   const gchar  *docpath,
+		   const gchar  *section,
+		   GError      **error)
+{
+	GIOChannel *io_channel = NULL;
+	ReadData   *read_data;
+
+	d(g_print ("Opening html-file: %s\n", docpath));
+
+	io_channel = g_io_channel_new_file (docpath, "r", error);
+
+	if (!io_channel) {
+		return;
+	}
+
+	read_data = g_new (ReadData, 1);
+	read_data->html = html;
+	read_data->stream = stream;
+	read_data->section = g_strdup (section);
+	
+	g_io_add_watch (io_channel, G_IO_IN | G_IO_HUP,
+			(GIOFunc) yelp_html_io_watch_cb,
+			read_data);
 }
 
 static void
@@ -332,24 +417,34 @@ yh_url_requested_cb (HtmlDocument *doc,
 		     HtmlStream   *stream,
 		     gpointer      data)
 {
+	YelpHtmlPriv     *priv;
         YelpHtml         *html;
 	GnomeVFSHandle   *handle;
 	GnomeVFSResult    result;
 	gchar             buffer[BUFFER_SIZE];
 	GnomeVFSFileSize  read_len;
-
-	d(g_print ("URL REQUESTED: %s\n", uri));
+	gchar            *abs_url;
+	gchar            *docpath;
 
         html = YELP_HTML (data);
-
+	priv = html->priv;
+	
 	html_stream_set_cancel_func (stream, yh_stream_cancel, html);
 
-	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
+	abs_url = yelp_util_resolve_relative_uri (priv->base_uri, uri);
+	docpath = yelp_util_extract_docpath_from_uri (abs_url);
 	
+	d(g_print ("URL REQUESTED: %s\n", abs_url));
+	g_free (abs_url);
+
+	result = gnome_vfs_open (&handle, docpath, GNOME_VFS_OPEN_READ);
+
 	if (result != GNOME_VFS_OK) {
-		g_warning ("Failed to open: %s", uri);
+		g_warning ("Failed to open: %s", docpath);
+		g_free (docpath);
 		return;
 	}
+	g_free (docpath);
 
 	while (gnome_vfs_read (handle, buffer, BUFFER_SIZE, &read_len) ==
 	       GNOME_VFS_OK) {
@@ -357,8 +452,6 @@ yh_url_requested_cb (HtmlDocument *doc,
 	}
 
 	gnome_vfs_close (handle);
-
-/*  	yelp_html_do_non_docbook (html, stream, uri, FALSE); */
 }
 
 static void
@@ -374,24 +467,39 @@ yh_stream_cancel (HtmlStream *stream,
 static void
 yh_link_clicked_cb (HtmlDocument *doc, const gchar *url, YelpHtml *html)
 {
-	gboolean handled;
+	YelpHtmlPriv *priv;
+	gboolean      handled;
+	gchar        *abs_url;
+	gchar        *section;
+	gchar        *docurl;
 
 	g_return_if_fail (HTML_IS_DOCUMENT (doc));
 	g_return_if_fail (url != NULL);
 	g_return_if_fail (YELP_IS_HTML (html));
 
+	priv    = html->priv;
 	handled = FALSE;
 
+	abs_url = yelp_util_resolve_relative_uri (priv->base_uri, url);
+	docurl  = yelp_util_split_uri (abs_url, &section);
+
+	d(g_print ("link clicked: ABS Url: %s\n", docurl));
+	
 	/* If this is a relative reference. Shortcut reload. */
-	if (url && (url[0] == '#' || url[0] == '?')) {
-		html_view_jump_to_anchor (HTML_VIEW (html->priv->view),
- 					  &url[1]);
-		handled = TRUE;
+	if (!g_ascii_strcasecmp (docurl, priv->base_uri) &&
+	    !strchr (abs_url, '?')) {
+		if (section) {
+			html_view_jump_to_anchor (HTML_VIEW (html->priv->view),
+						  section);
+			handled = TRUE;
+		}
 	}
+
+	g_free (abs_url);
 	
 	d(g_print ("link clicked: URL=%s baseUri=%s\n", url,
 		   html->priv->base_uri));
-
+	
 	g_signal_emit (html, signals[URL_SELECTED], 0,
 		       url, html->priv->base_uri, handled);
 }
@@ -423,13 +531,13 @@ yelp_html_new (void)
 void
 yelp_html_open_uri (YelpHtml     *html, 
 		    const gchar  *str_uri,
-		    const gchar  *reference,
+		    const gchar  *section,
 		    GError      **error)
 {
         YelpHtmlPriv *priv;
 	GdkCursor    *cursor;
 	
-	d(g_print ("Open URI: %s\n", str_uri));
+	d(g_print ("Open URI: %s with section: %s\n", str_uri, section));
 
 	g_return_if_fail (YELP_IS_HTML (html));
 	g_return_if_fail (str_uri != NULL);
@@ -453,14 +561,43 @@ yelp_html_open_uri (YelpHtml     *html,
 	gdk_window_set_cursor (GTK_WIDGET (priv->view)->window, cursor);
 	gdk_cursor_unref (cursor);
 	
-	if (!strncmp (str_uri, "ghelp:", 6)) {
-		yelp_html_do_docbook (html, str_uri + 6, error);
-		/* Docbook or HTML */
-		return;
-	}
+	g_free (priv->base_uri);
+	priv->base_uri = g_strdup (str_uri);
 
-	yelp_html_do_non_docbook (html, priv->doc->current_stream, 
-				  str_uri, TRUE, error);
+	if (!strncmp (str_uri, "man:", 4)) {
+		yelp_html_do_maninfo (html, priv->doc->current_stream, MAN,
+				      str_uri + 4, error);
+	}
+	else if (!strncmp (str_uri, "info:", 5)) {
+		yelp_html_do_maninfo (html, priv->doc->current_stream, INFO,
+				      str_uri + 5, error);
+	} else {
+		gchar *mime_type = NULL;
+		gchar *docpath;
+
+		docpath   = yelp_util_extract_docpath_from_uri (str_uri);
+		mime_type = gnome_vfs_get_mime_type (docpath);
+
+		if (mime_type) {
+			d(g_print ("mime_type: %s\n", mime_type));
+			
+			if (!g_strcasecmp (mime_type, "text/sgml") ||
+			    !g_strcasecmp (mime_type, "text/xml")) {
+				yelp_html_do_docbook (html,
+						      docpath, section, error);
+			}
+			else if (!g_strcasecmp (mime_type, "text/html")) {
+				yelp_html_do_html (html,
+						   priv->doc->current_stream,
+						   docpath, section, error);
+			}
+
+			g_free (mime_type);
+		}
+
+		d(g_print ("Docpath: %s\n", docpath));
+		g_free (docpath);
+	}
 }
 
 void
